
<template>
    <div class="s-report-pane">
        <div v-if="error" class="s-report-error">
            {{error}}
        </div>
        <div v-if="analysis" class="s-report-section">
            <div style="float:right; margin-top: 10px">
                <b-btn v-if="noConfirmDelete" variant="outline-secondary" @click="okDelete">Delete</b-btn>
                <b-btn v-else variant="outline-secondary" v-b-modal.modal3>Delete</b-btn>
            </div>
            <h3>Analysis:</h3>
            <table>
                <tr>
                    <td rowspan="3"><a :href="'api/analyses/' + id + '/PDF?file=report.pdf'"><img src="static/images/pdf.png" class="s-pdf-icon"></a></td>
                    <td>Not attributed: {{analysis.ReadComposition.NotAttributed}}</td></tr>
                <tr><td>Not informative (low complexity etc): {{analysis.ReadComposition['Non-informative']}}</td></tr>
                <tr><td>Human: {{analysis.ReadComposition.Human}}</td></tr>
            </table>
            <b-modal id="modal3" title="Please confirm" size="sm" @ok="okDelete">
                <b-container fluid>
                    Delete selected analysis?
                    <div style="margin: 20px 0; color:#888">
                        <b-form-checkbox v-model="noConfirmDelete">Don't ask for the next 24h</b-form-checkbox>
                    </div>
                </b-container>
            </b-modal>
        </div>

        <div v-if="summary" class="s-report-section">
            <h3>Outlook:</h3>
            <table class="s-outlook-table">
                <tr>
                    <th></th>
                    <th>N. of <br> species</th>
                    <th>Possibly medically relevant</th>
                </tr>
                <tr v-for="(data, key) in summary.Summary">
                    <td>{{key}}</td>
                    <td style="text-align: center">{{data.NbSpecies}}</td>
                    <td>
                        <table v-if="data.Relevant && data.Relevant.length">
                            <tr v-for="item in data.Relevant">
                                <td>{{item.Species}}</td>
                                <td>confidence: {{item.Confidence}}</td>
                                <td>load: {{item.Load}}</td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </div>

        <div v-if="results" class="s-report-section">

            <div class="row">
                <div class="col-md-6">
                    <h3>List of findings:</h3>
                </div>
                <div class="col-md-5">
                    <b-form-fieldset horizontal label="Filter" :label-cols="3">
                        <b-form-input v-model="filter" placeholder="Type to Search" />
                    </b-form-fieldset>
                </div>
                <div class="col-md-1">
                    <b-btn v-b-modal.modal2>...</b-btn>
                </div>
            </div>

            <b-table small hover :items="results.rows || results.report" :fields="columns" :filter="filter">
                <template #cell(Output)="data">
                  <a :href="coverageUrl(id, data)" target="_blank" v-b-popover="coveragePopup(id, data)">coverage</a>,
                  <a :href="readsUrl(id, data)" target="_blank"">reads</a>
                </template>
            </b-table>

            <b-modal id="modal2" title="Select Columns" size="sm" @ok="okModal">
                <b-container fluid>
                    <div v-for="col in initial">
                        <b-form-checkbox v-model="col.selected">{{col.label}}</b-form-checkbox>
                    </div>
                </b-container>
            </b-modal>
        </div>

        <div v-if="noresults" class="s-report-section">

            <div class="row">
                <div>The specimen was NEGATIVE for all microorganisms</div>
            </div>

        </div>

        <img v-else :src="'api/analyses/' + id + '/PLOT'" class="s-report-chart"></img>
    </div>
</template>

<style>

.s-report-pane {
    overflow: auto;
    flex: 1;
    padding: 0 50px;
    border-left: 1px solid #ddd;
}

.s-pdf-icon {
    width: 60px;
    margin: 10px 20px;
}

.s-report-section {
    margin-bottom: 2em;
}

.s-report-chart {
    width: 100%;
    max-width: 50em;
}

.s-report-section>table {
    white-space: nowrap;
}

.s-outlook-table {
    line-height: 1.2em;
    border-collapse: collapse;
}

.s-outlook-table th,
.s-outlook-table td {
    padding: 5px 10px;
    border: 1px solid #eee;
}

.s-outlook-table table td {
    padding: 0 5px;
    border: none;
}


</style>

<script>

var axios = require('axios');


function data(){

    var noConfirmDelete;

    try {
        var confirmDate = localStorage.getItem('confirm-delete');
        noConfirmDelete = (confirmDate > Date.now());
    }
    catch(err){
        // ignore
    }

    return {
        results: null,
        noresults: false,
        summary: null,
        analysis: null,
        loading: false,
        error: '',
        columns: [],
        initial: [],
        filter: '',
        noConfirmDelete: noConfirmDelete
    };
}


function created(){
    onID.call(this, this.id);
}


function selected(col){
    return col.selected;
}


function loadSelected(){

    try {

        var selected = {}, columns = [], stored = localStorage.getItem('report-columns');

        if (stored) {
            columns = JSON.parse(stored);
        }
        else {
            return null;
        }

        columns.forEach(function(col){
            selected[col.key] = col.selected;
        });

        return selected;
    }
    catch(err){
        return null;
    }
}


function getColumns(){

    if (!this.results){
        return;
    }

    var i, columns = [], items = this.results.rows || this.results.report, sel = loadSelected();

    for(i in items[0]){
        columns.push({
            key: i,
            label: i,
            sortable: true,
            selected: sel ? sel[i] || false : true
        });
    }

    this.initial = columns;
    this.columns = columns.filter(selected);
}


function onID(id){

    var self = this;

    if (!id || id == '-'){
        self.loading = false;
        self.results = null;
        self.summary = null;
        return;
    }

    self.loading = true;

    function load(name, property){
        return axios.get('api/analyses/' + id + '/JSON?file=' + name).then(function(response){
            if (property == 'results') {
                var rows = response.data && response.data.rows;
                if (!rows || rows[0] == 'no results' || (rows[0] && rows[0].TaxID == 'N.A.')){
                    self.noresults = true;
                    self.results = null;
                }
                else {
                    self.noresults = false;
                    self.results = response.data;
                }
            }
            else {
                self[property] = response.data;
            }
        });
    }

    var files = [
        load('results.json', 'results').then(getColumns.bind(self)),
        //load('summary.json', 'summary'),
        load('analysis.json', 'analysis')
    ];

    function byTotalNtCovered(r1, r2){

        var n = 'Total Nt Covered',
            v1 = parseFloat(r1[n]),
            v2 = parseFloat(r2[n]);

        if (v1 > v2) {
            return -1;
        }

        if (v1 < v2) {
            return 1;
        }

        return 0;
    }

    function ok(response){
        self.loading = false;
        if (self.results && Array.isArray(self.results.rows)){
            self.results.rows.sort(byTotalNtCovered);
        }
    }

    function err(reason){
        self.loading = false;
        self.error = reason.message;
    }

    Promise.all(files).then(ok, err);
}



function saveConfirmDelete(value){

    try {
        localStorage.setItem('confirm-delete', value ? Date.now() + 24 * 60 * 60 * 1000 : 0);
    }
    catch(err){
        // ignore
    }
}


var watch = {
    id: onID,
    noConfirmDelete: saveConfirmDelete
};


var methods = {

    coverageUrl: function(id, data){
        return 'api/analyses/' + id + '/PLOT?taxid=' + data.item.TaxID;
    },

    readsUrl: function(id, data){
        return 'api/analyses/' + id + '/READS?taxid=' + data.item.TaxID;
    },

    coveragePopup: function(id, data){

        var url = this.coverageUrl(id, data),
            content = '<img src="' + url + '" width="150px">';

        return {
            html: true,
            trigger: 'hover',
            placement: 'left',
            content: content
        };
    },

    okModal: function(){
        this.columns = this.initial.filter(selected);
        localStorage.setItem('report-columns', JSON.stringify(this.initial));
    },

    okDelete: function(){
        this.$router.push('/explorer');
        this.$root.deleteAnalysis(this.id);
    }
};

module.exports = {
    props: ['id'],
    data: data,
    watch: watch,
    methods: methods,
    created: created
};


</script>